(* $Id *)

signature ACLRIGHTS =
sig
  type rights

  exception InvalidRights

  val ACLx : rights
  val ACLr : rights
  val ACLw : rights
  val ACLi : rights
  val ACLa : rights
  val empty : rights

  val has : rights * rights -> bool

  val union : rights * rights -> rights

  val diff : rights * rights -> rights

  val toString : rights -> string

  val fromString : string -> rights
end

signature ACL =
sig
  structure Auth : AUTH
  structure Rights : ACLRIGHTS

  exception SyntaxError of string

  type acl

  val empty : acl

  val toString : acl -> string
  val optToString : acl option -> string
  val toValue : acl -> Value.value

  (* these can raise SyntaxError *)
  val fromString : string -> acl
  val fromValue : Value.value -> acl

  val update : acl -> string * Rights.rights -> acl

  val union : acl * acl -> acl

  val myrights : Auth.ident -> acl -> Rights.rights

  val fmt : acl -> string

  val scan : (char, 'strm) StringCvt.reader -> (acl, 'strm) StringCvt.reader
end
