(* $Id: dataset.sig,v 1.1.1.1 1999/12/15 01:15:54 leg Exp $ *)

signature DATASET =
sig
  structure T : THROTTLE
  structure AcapTime : ACAPTIME
  structure Entry : ENTRY

  (* type dataset *)

  (* --- datasets --- *)
  exception NoSuchDataset
  exception DatasetRefer of string list
  exception SyntaxError of string
  exception DiskError of string

  val init : unit -> unit

  (* search the named dataset 
   params: the user,
           the named dataset (slash terminated), the depth, inheritance,
           the search function, 
	   a messenger to leave at each dataset to be called,
	   the throttle to send output to, where to send referrals
	   output comes in (path : string, entry) form. *)

  val search : Entry.Acl.Auth.ident -> (string * int * bool)
    -> (Entry.entry -> bool)
    -> (string * Entry.updatemsg RMulticast.port -> unit) option 
    -> (string * string * Entry.entry -> unit) * 
       ((string * string list) -> unit)
    -> (unit -> unit)
    -> unit

  (* store the list of entry/attribute pairs in this dataset;
   entries should be _full_ name
   params: the user,
           the dataset, whether to create or not, time to abort,
           what to store
           *)
  val store : Entry.Acl.Auth.ident -> string * bool * AcapTime.acaptime option
    -> (string * Entry.storedata list)
    -> unit

  (* acl stuff *)
  type aclobj = {dataset : string,
		 attribute : string option,
		 entry : string option}

  datatype aclmod = 
    SETACL of string * Entry.Acl.Rights.rights
  | DELACL of string option
  | GETACL

  val modifyacl : Entry.Acl.Auth.ident -> (aclobj * aclmod) -> Entry.Acl.acl

end
